import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, Date as ZdDate, DateRange, DialogService, Form, LoadingService, Modal, ModalService, Select,
    Button,
    SelectMultiple, TextInput, Time,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Loader, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

interface RowTarefas {
    CDCLIENTE: string;
    CDOPERADOR: string;
    CDOPERADORPAI: string;
    DTHORAAGEND: string;
    DTHRFINALAGEND: string;
    IDATIVO: string;
    IDOBRIGAOBSERVACAO: string;
    IDSITUACAOAGEND: string;
    IDTIPOMARCACAOTAREFA: string;
    IDTIPORECORRENCIA: string;
    NMCHECKLIST: string;
    NMCHECKLISTAGEND: string;
    NMFANTCLIE: string;
    NMLOCALCLIENTE: string;
    NMOPERADOR: string;
    NMRAZSOCCLIE: string;
    NMTIPOLOCALCLI: string;
    NRATIVIDADECHK: number;
    NRCHECKLIST: number;
    NRCHECKLISTAGEND: number;
    NRCONTRATOFCT: string;
    NRCRITERIOTAREFACHK: string;
    NRESTRUTURAM: number;
    NRLOCALCLIENTE: number;
    NRPRIORIDADETAREFA: number;
    NRSEQCONTRATO: number;
    NRTAREFACHK: string;
    NRTIPOLOCALCLI: number;
}

export class Fm00004000TarefasController {
    private cadastroTarefaUnica!: Modal;

    private cadastroTarefaModal!: Modal;

    private exibirInformacoesTarefaModal!: Modal;

    private basicModalCriterio!: Modal;

    public visibilidadeGridItens: boolean = false;

    public visibilidadeForm: boolean = true;

    public parametrizacao = '';

    public row: any = [];

    private tarefaMassaController = Loader.getInstance('Fm00004000TarefasMassaController');

    public habilitaOpcaoDias: boolean = true;

    // ============Controle do modal==========
    // O modal cadastroTarefas  também é usado na tela 'Calendário' pelo botão 'Cadastrar Tarefa'teste
    public async exibirModalCadastroTarefas({ event }: any) {
        event.preventDefault();
        this.cadastroTarefaUnica = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_001_modal_cadastro_tarefas' });
        this.cadastroTarefaUnica.show();
        this.validaParametrizacao();
        this.habilitaOpcaoDias = (this.parametrizacao === 'N');
    }

    public async exibirModalInformacoesTarefas({ row, event }: { row: RowTarefas, event: any }): Promise<void> {
        event.preventDefault();
        this.criarModal();
        setTimeout(() => {
            this.popularModalInforamcoesTarefas(row);
        }, 2500);
    }

    private setValueSelectCamposModal<T extends Select | SelectMultiple>(field: string, props: T | T[] | string | number | number[]): void {
        Metadata.getInstance<T>(field).setValue(props, false);
    }

    private popularModalInforamcoesTarefas(row: RowTarefas): void {
        Metadata.getInstance<TextInput>('informacoesNomeTarefa').value = row.NMCHECKLISTAGEND;
        this.setValueSelectCamposModal<Select>('informacoesNomeCliente', row.CDCLIENTE);
        this.changeCampoChecklist(row);
        this.changeCampoTipoDeLocal(row);
        this.changeCampoLocalModalInformacoes(row);
        this.changeItensChecklistModalInformacoes(row);
        this.changeEscopoModalInformacoes(row);
        this.changeAtividadeModalInformacoes(row);
        this.changeContratosModalInformacoes(row);
        this.changeFuncionarioModalInformacoes(row);
        this.changeEncarregadoModalInformacoes(row);
        this.setDataTarefaModalInformacoes(row);
        this.setHorarioTarefaModalInformacoes(row);
        this.setDuracaodaTarefaModalInformacoes(row);
        this.setObervacoesObrigatorias(row);
        this.changeCheckListModalInformacoes(row);
        this.setValueSelectCamposModal<Select>('select-Informacoes-prioridade', row.NRPRIORIDADETAREFA);
        this.setValueSelectCamposModal<Select>('selectInformacoesOpcoesMarcacao', row.IDTIPOMARCACAOTAREFA);
    }

    private setDataTarefaModalInformacoes(row: RowTarefas): void {
        const data = row.DTHORAAGEND.slice(0, 10);
        Metadata.getInstance<ZdDate>('Informacoes-DTHORAAGEND').value = data;
    }

    private setHorarioTarefaModalInformacoes(row: RowTarefas): void {
        const horario = row.DTHORAAGEND.slice(11, 16);
        Metadata.getInstance<Time>('selectInformacoesHoraInicio1').value = horario;
    }

    private setObervacoesObrigatorias(row: RowTarefas): void {
        const observacoes = Metadata.getInstance<Select>('selectInformacoesObservacoesObrigatorias');

        if (row.IDOBRIGAOBSERVACAO === 'N') {
            this.setValueSelectCamposModal<Select>(observacoes.name, '1');
        } else {
            this.setValueSelectCamposModal<Select>(observacoes.name, '2');
        }
    }

    private setDuracaodaTarefaModalInformacoes(row: RowTarefas): void {
        const horarioInicialString: string = row.DTHORAAGEND.slice(11, 16);
        const horarioFinalString: string = row.DTHRFINALAGEND.slice(11, 16);

        const horarioInicial: Date = new Date(`1970-01-01T${horarioInicialString}:00`);
        const horarioFinal: Date = new Date(`1970-01-01T${horarioFinalString}:00`);

        const diferencaMinutos: number = (horarioFinal.getTime() - horarioInicial.getTime()) / (1000 * 60);

        const duracaoTarefa = diferencaMinutos.toString();

        Metadata.getInstance<Time>('selectInformacoesdDuracao1').value = (`00:${duracaoTarefa}`);
    }

    private changeCampoTipoDeLocal(row: RowTarefas): void {
        const tipoLocal = Metadata.getInstance<Select>('informacoesTipoLocalCli');
        tipoLocal.datasource.setFilter({
            CDCLIENTE: row.CDCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(tipoLocal.name, row.NRTIPOLOCALCLI);
    }

    private changeCampoChecklist(row: RowTarefas): void {
        const checklist = Metadata.getInstance<Select>('informacoesNRCHECKLIST');
        checklist.datasource.setFilter({
            NRTIPOLOCALCLI: row.NRTIPOLOCALCLI,
            UNICA: 'S',
        });
        this.setValueSelectCamposModal<Select>(checklist.name, row.NRCHECKLIST);
    }

    private changeCampoLocalModalInformacoes(row: RowTarefas): void {
        const local = Metadata.getInstance<Select>('informacoesLocalCli');
        local.datasource.setFilter({
            NRTIPOLOCALCLI: row.NRTIPOLOCALCLI,
            UNICA: 'S',
            CDCLIENTE: row.CDCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(local.name, row.NRLOCALCLIENTE);
    }

    private changeItensChecklistModalInformacoes(row: RowTarefas): void {
        const itenschecklist = Metadata.getInstance<SelectMultiple>('informacoesItensChecklist');
        itenschecklist.datasource.setFilter({
            NRCHECKLIST: row.NRCHECKLIST,
        });
        this.setValueSelectCamposModal<SelectMultiple>(itenschecklist.name, [row.NRATIVIDADECHK]);
    }

    private changeEscopoModalInformacoes(row: RowTarefas): void {
        const escopo = Metadata.getInstance<SelectMultiple>('informacoesEscopoTarefa');
        escopo.datasource.setFilter({
            NRTAREFACHK: row.NRATIVIDADECHK,
        });
        this.setValueSelectCamposModal<SelectMultiple>(escopo.name, [row.NRATIVIDADECHK]);
    }

    private changeAtividadeModalInformacoes(row: RowTarefas): void {
        const atividade = Metadata.getInstance<SelectMultiple>('informacoesAtividadeTarefa');
        atividade.datasource.setFilter({
            NRATIVIDADECHK: [row.NRATIVIDADECHK],
        });
        const arrayNrtarefachk: number[] = row.NRTAREFACHK.split(', ').map(Number);
        this.setValueSelectCamposModal<SelectMultiple>(atividade.name, arrayNrtarefachk);
    }

    private changeCheckListModalInformacoes(row: RowTarefas): void {
        const arrayNrtarefachkString = row.NRTAREFACHK.split(', ').map(String);
        const checkbox = Metadata.getInstance<CheckboxMultiple>('informacoes-NRCRITERIOTAREFACHK');
        checkbox.datasource?.setFilter({
            NRTAREFACHK: arrayNrtarefachkString,
            NRATIVIDADECHK: [row.NRATIVIDADECHK],
            NRCHECKLIST: row.NRCHECKLIST,
        });
        const arrayNrCriterioTarefaCHK = row.NRCRITERIOTAREFACHK.split(', ').map(String);
        checkbox.value = arrayNrCriterioTarefaCHK;
    }

    private changeContratosModalInformacoes(row: RowTarefas): void {
        const selectContratos = Metadata.getInstance<Select>('informacoes-NRSEQCONTRATO');
        selectContratos.datasource.setFilter({
            NRESTRUTURAM: row.NRESTRUTURAM,
        });
        this.setValueSelectCamposModal<Select>(selectContratos.name, row.NRSEQCONTRATO);
    }

    private changeFuncionarioModalInformacoes(row: RowTarefas): void {
        const funcionario = Metadata.getInstance<Select>('selectInformacoesFuncionario1');
        funcionario.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
        });
        this.setValueSelectCamposModal<Select>(funcionario.name, row.CDOPERADOR);
    }

    private changeEncarregadoModalInformacoes(row: RowTarefas): void {
        const encarregado = Metadata.getInstance<Select>('selectInformacoesEncarregado1');
        encarregado.datasource.setFilter({
            NRLOCALCLIENTE: row.NRLOCALCLIENTE,
            CDOPERADOR: row.CDOPERADOR,
        });

        this.setValueSelectCamposModal<Select>(encarregado.name, row.CDOPERADORPAI);
    }

    public async criarModal(): Promise<void> {
        this.exibirInformacoesTarefaModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_001_modal_tarefas' });
        this.exibirInformacoesTarefaModal.show();
    }

    public destruirModal() {
        this.cadastroTarefaModal?.destroy();
    }

    public async exibirModalTarefas({ row }: any): Promise<void> {
        this.row = row;
        this.cadastroTarefaModal.show();
        setTimeout(() => {
            this.popularModalTarefas();
        }, 500);
    }

    public salvarInformacoesModalEdicaoTarefas(): void {
        // const formTarefas = Metadata.getInstance<ZdForm>('formCadastroTarefasModal');
        // const formCriterios = Metadata.getInstance<ZdForm>('formTarefas_informacoes-dropBoxCriterioAvaliacao');
        console.warn([
            'NOMETAREFA =', Metadata.getInstance<TextInput>('informacoesNomeTarefa').value,
            'CDCLIENTE =', Metadata.getInstance<Select>('informacoesNomeCliente').value,
            'NRTIPOLOCALCLI =', Metadata.getInstance<SelectMultiple>('informacoesTipoLocalCli').value,
            'NRLOCALCLIENTE =', Metadata.getInstance<SelectMultiple>('informacoesLocalCli').value,
            'NRCHECKLIST =', Metadata.getInstance<Select>('informacoesNRCHECKLIST').value,
            'NRATIVIDADECHK =', Metadata.getInstance<SelectMultiple>('informacoesItensChecklist').value,
            'NRESCOPOSTAREFA =', Metadata.getInstance<SelectMultiple>('informacoesEscopoTarefa').value,
            'NRTAREFACHK =', Metadata.getInstance<SelectMultiple>('informacoesAtividadeTarefa').value,
            // NRCRITERIOTAREFACHK: checkBoxMultiple.datasource?.data.filter((box: any) => checkBoxMultiple.value.indexOf(box.NRCRITERIOTAREFACHK) >= 0),
            // DATERANGE: dataRange,
            // DIASEMANA: diaSemana,
            'VISTORIA =', Metadata.getInstance<SelectMultiple>('selectInformacoesCriarVistoria').value,
            'OBSERVACOES =', Metadata.getInstance<Select>('selectInformacoesObservacoesObrigatorias').value,
            'NRPRIORIDADETAREFA =', Metadata.getInstance<Select>('select-Informacoes-prioridade').value,
        ]);
        // const formAgendamento = Metadata.getInstance<ZdForm>('formTarefas_dropBoxInformacoesDadosAgendamento');
        // if (formTarefas && formCriterios && formAgendamento) {
        //     LoadingService.show();
        //     Http.post('/verificarOcorrencias', {
        //         NOMETAREFA: Metadata.getInstance<TextInput>('informacoesNomeTarefa').value,
        //         CDCLIENTE: Metadata.getInstance<Select>('informacoesNomeCliente').value,
        //         NRTIPOLOCALCLI: Metadata.getInstance<SelectMultiple>('informacoesTipoLocalCli').value,
        //         NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('informacoesLocalCli').value,
        //         NRCHECKLIST: Metadata.getInstance<Select>('informacoesNRCHECKLIST').value,
        //         NRATIVIDADECHK: Metadata.getInstance<SelectMultiple>('informacoesItensChecklist').value,
        //         NRESCOPOSTAREFA: Metadata.getInstance<SelectMultiple>('informacoesEscopoTarefa').value,
        //         NRTAREFACHK: Metadata.getInstance<SelectMultiple>('informacoesAtividadeTarefa').value,
        //         // NRCRITERIOTAREFACHK: checkBoxMultiple.datasource?.data.filter((box: any) => checkBoxMultiple.value.indexOf(box.NRCRITERIOTAREFACHK) >= 0),
        //         // DATERANGE: dataRange,
        //         // DIASEMANA: diaSemana,
        //         VISTORIA: Metadata.getInstance<SelectMultiple>('selectInformacoesCriarVistoria').value,
        //         OBSERVACOES: Metadata.getInstance<Select>('selectInformacoesObservacoesObrigatorias').value,
        //         NRPRIORIDADETAREFA: Metadata.getInstance<Select>('select-prioridade').value,
        //     }).then((response: IDictionary) => {
        //         if (response.data !== 'OK') {
        //             this.tarefaMassaController.abrirModalConflito(response.data);
        //             this.fecharModalCadastroTarefas();
        //         } else {
        //             Utilities.showSuccessMessage();
        //             this.tarefaMassaController.fecharModalConflito();
        //             this.fecharModalCadastroTarefas();
        //             // grid.datasource.get();
        //         }
        //     }).catch((error: any) => {
        //         Utilities.showErrorMessage(error);
        //     }).finally(() => {
        //         LoadingService.hide();
        //     });
        // }
    }

    public popularModalTarefas() {
        Http.post('/getDadosModalTarefas', {
            NRCHECKLISTAGEND: this.row.NRCHECKLISTAGEND,
            NRATIVIDADECHK: this.row.NRATIVIDADECHK,
            NRCHECKLIST: this.row.NRCHECKLIST,
        }).then((response: any) => {
            const [data, hora] = this.row.DTHORAAGEND.split(' ');
            Metadata.getInstance<TextInput>('atividadeTarefa').value = response.data.ATIVIDADE;
            Metadata.getInstance<TextInput>('nomeTarefa').value = this.row.NMCHECKLISTAGEND;
            Metadata.getInstance<TextInput>('nomeCliente').value = this.row.NMFANTCLIE;
            Metadata.getInstance<TextInput>('tipoLocalCli').value = this.row.NMTIPOLOCALCLI;
            Metadata.getInstance<TextInput>('localCli').value = this.row.NMLOCALCLIENTE;
            Metadata.getInstance<TextInput>('NRSEQCONTRATO').value = this.row.NRCONTRATOFCT;
            Metadata.getInstance<Select>('NRCHECKLIST').value = this.row.NMCHECKLIST;
            Metadata.getInstance<TextInput>('selectData').value = data;
            Metadata.getInstance<TextInput>('selectHoraInicio1').value = hora;
            Metadata.getInstance<TextInput>('selectFuncionario1').value = this.row.NMOPERADOR;
            Metadata.getInstance<Select>('selectObservacoesObrigatorias').value = this.row.IDOBRIGAOBSERVACAO;
            Metadata.getInstance<Select>('IDTIPORECORRENCIA').value = this.row.IDTIPORECORRENCIA;
            Metadata.getInstance<Select>('selectEncarregado1').value = this.row.CDOPERADORPAI;
            Metadata.getInstance<Select>('escopoTarefa').value = this.row.NMESCOPOSTAREFA;
            Metadata.getInstance<Select>('itensChecklist').value = this.row.NMATIVIDADE;
            Metadata.getInstance<Select>('selectOpcoesMarcacao').value = this.row.IDTIPOMARCACAOTAREFA;
            Metadata.getInstance<TextInput>('selectdDuracao1').value = this.formatarDataHora(this.row.DTHORAAGEND.split(' '), this.row.DTHRFINALAGEND.split(' '));
            this.popularCriteriosModal(this.row.NRCHECKLIST, this.row.NRATIVIDADECHK, response.data.CRITERIOS[0].NRTAREFACHK);
            Metadata.getInstance<TextInput>('escopoTarefa').value = response.data.ESCOPO;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public populaEscopo() {
        const escopoServico = Metadata.getInstance<SelectMultiple>('NRESCOPOSTAREFA');
        Http.post('/populaEscopo', { NRTAREFACHK: Metadata.getInstance<Select>('NRTAREFACHK').value }).then((response: any) => {
            escopoServico.value = response.data.data;
        });
    }

    public populaCriterios() {
        const tarefa = Metadata.getInstance<SelectMultiple>('NRTAREFACHK');
        const itenschecklist = Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        const checkbox = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        if (tarefa.value.length > 0) {
            checkbox.datasource?.setFilter({
                NRTAREFACHK: tarefa.value,
                NRATIVIDADECHK: itenschecklist.value,
                NRCHECKLIST: checklist.value,
            });
            checkbox.datasource?.get();
            checkbox.isVisible = true;
            Http.post('/populaCriterios', { NRTAREFACHK: Metadata.getInstance<Select>('NRTAREFACHK').value }).then((response: any) => {
                checkbox.value = response.data;
            });
        } else {
            tarefa.setValue([]);
            checkbox.isVisible = false;
        }
    }

    public async fecharModalInformacoesTarefas() {
        this.exibirInformacoesTarefaModal.hide();
        this.exibirInformacoesTarefaModal?.destroy();
    }

    public async fecharModalCadastroTarefas() {
        this.cadastroTarefaUnica.hide();
        this.cadastroTarefaUnica?.destroy();
    }

    public async exibirModalCriterios() {
        this.basicModalCriterio = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/fm_00004_002_modal_cadastro_criterio' });
        this.basicModalCriterio.show();
    }

    public async fecharModalCriterio() {
        this.basicModalCriterio.hide();
        this.basicModalCriterio?.destroy();
    }

    public clickButtonToExpand({ component }: any) {
        const form = Metadata.getInstance<Form>(`formTarefas_${component.name}`);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    // ==========Controles dos Selects ===================

    // INTEGRAÇÃO COM SM
    public changeContratos({ component }: any) {
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE').value;
        const selectContratos = Metadata.getInstance<Select>('NRSEQCONTRATO');
        const encarregado = Metadata.getInstance<Select>('selectEncarregado1');
        const funcionario = Metadata.getInstance<Select>('selectFuncionario1');

        if (component.value > 0) {
            selectContratos.disabled = false;
            const { selectValue } = component;
            selectContratos.datasource.setFilter({
                NRESTRUTURAM: selectValue.NRESTRUTURAM,
            });

            encarregado.datasource.setFilter({
                NRLOCALCLIENTE: local,
            });

            funcionario.datasource.setFilter({
                NRLOCALCLIENTE: local,
            });
        } else if (component.value < 1) {
            selectContratos.disabled = true;
            selectContratos.value = null;
        }
    }

    public async setFilterEncarregado({ component }: IEventParam<Select>) {
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE').value;
        const encarregado = Metadata.getInstance<Select>('selectEncarregado1');
        const criaVistoria = Metadata.getInstance<Select>('selectCriarVistoria');
        if (component.value) {
            encarregado.disabled = false;

            await encarregado.datasource.setFilter({
                NRLOCALCLIENTE: local,
                CDOPERADOR: component.value,
            });

            setTimeout(() => {
                if (encarregado.datasource.data.length <= 0 && criaVistoria.value === '1') {
                    const message = `O operador ${component.selectValue.NMOPERADOR} é o usuário máximo da Hierarquia atual. Gostaria de proceder com a criação de uma vistoria referente a esta tarefa para ele?`;
                    this.dialogConfirmarCriacaoVistoria(message);
                }
            });
        } else {
            encarregado.disabled = true;
        }
    }

    public dialogConfirmarCriacaoVistoria(msg: string) {
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Atenção!!',
            text: msg,
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: '{{Fm00004000TarefasController.acaoDialog}}',
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: '{{Fm00004000TarefasController.acaoDialog}}',
                    },
                },
            ],
        });
    }

    public acaoDialog({ component }: IEventParam<Button>) {
        const encarregado = Metadata.getInstance<Select>('selectEncarregado1');
        const criaVistoria = Metadata.getInstance<Select>('selectCriarVistoria');

        // Significa que o usuário não deseja continuar com a criação da vistoria para um Operador Máximo na Hierarquia
        if (component.label === 'Não') {
            criaVistoria.value = '2';
        }

        encarregado.removeValidation('required');
        encarregado.disabled = true;
        DialogService.hide();
    }

    public focusOperadores({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-dados-agendamento');
        if (grid.editing) {
            component.datasource.setFilter({
                NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
            });
        }
    }

    public changeTipoDeLocal({ component }: any) {
        const tipoLocal = Metadata.getInstance<Select>('NRTIPOLOCALCLI');
        if (component.value) {
            tipoLocal.disabled = false;
            tipoLocal.datasource.setFilter({
                CDCLIENTE: component.value,
            });
        } else {
            tipoLocal.disabled = true;
            tipoLocal.value = null;
        }
    }

    public changeLocal() {
        const tipoLocal = Metadata.getInstance<Select>('NRTIPOLOCALCLI');
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');

        if (tipoLocal.value > 0) {
            local.disabled = false;
            checklist.disabled = false;
            local.datasource.setFilter({
                NRTIPOLOCALCLI: tipoLocal.value,
                UNICA: 'S',
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
            });
            checklist.datasource.setFilter({
                NRTIPOLOCALCLI: tipoLocal.value,
                UNICA: 'S',
            });
        } else {
            local.value = null;
            local.disabled = true;
            checklist.value = null;
            checklist.disabled = true;
        }
    }

    public changeItensChecklist() {
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        const itenschecklist = Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK');
        if (checklist.value > 0) {
            itenschecklist.disabled = false;
            itenschecklist.datasource.setFilter({
                NRCHECKLIST: checklist.value,
            });
        } else {
            itenschecklist.setValue([]);
            itenschecklist.disabled = true;
        }
    }

    public changeEscopo() {
        const itenschecklist = Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI');
        const escopo = Metadata.getInstance<Select>('NRESCOPOSTAREFA');

        if (itenschecklist.value > 0) {
            escopo.disabled = false;
        } else {
            escopo.value = null;
            escopo.disabled = true;
        }
    }

    public changeTarefa() {
        const escopo = Metadata.getInstance<SelectMultiple>('NRESCOPOSTAREFA');
        const tarefa = Metadata.getInstance<SelectMultiple>('NRTAREFACHK');
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');
        if (escopo.value.length > 0) {
            tarefa.disabled = false;
            tarefa.datasource.setFilter({
                NRATIVIDADECHK: itenschecklist.value,
            });
        } else {
            tarefa.setValue([]);
            tarefa.disabled = true;
        }
    }

    public setCriterios() {
        const tarefa = Metadata.getInstance<SelectMultiple>('NRTAREFACHK');
        const itenschecklist = Metadata.getInstance<Select>('NRATIVIDADECHK');
        const checklist = Metadata.getInstance<Select>('NRCHECKLIST');
        const operador = Metadata.getInstance<Select>('selectFuncionario1');
        const local = Metadata.getInstance<Select>('NRLOCALCLIENTE');
        const checkbox = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        const prioridade = Metadata.getInstance<Select>('select-prioridade');

        if (tarefa.value.length > 0) {
            checkbox.datasource?.setFilter({
                NRTAREFACHK: tarefa.value,
                NRATIVIDADECHK: itenschecklist.value,
                NRCHECKLIST: checklist.value,
            });
            checkbox.datasource?.get();
            checkbox.isVisible = true;

            operador.datasource.setFilter({
                NRLOCALCLIENTE: local.value,
                NRTAREFACHK: tarefa.value,
            });

            // trecho de código adicionado para validação de prioridade
            const arrayPrioritario: any = tarefa.selectValue.find((array: any) => array.IDPRIORIDADE === 1);
            if (arrayPrioritario) {
                prioridade.disabled = true;
                prioridade.value = arrayPrioritario.NRPRIORIDADETAREFA;
            } else {
                prioridade.disabled = false;
                prioridade.value = null;
            }
        } else {
            tarefa.setValue([]);
            checkbox.isVisible = false;
        }
    }

    public novoCriterio() {
        const chk = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        const valor = Metadata.getInstance<TextInput>('inputNovoCriterio').value;
        chk!.datasource!.data!.push({
            NRCRITERIOTAREFACHK: valor,
            DSCRITERIOTAREFA: valor,
        });
        chk!.value.push(valor);
        this.fecharModalCriterio();
    }

    public setFiltroGridPrincipal({ component }: any) {
        component.datasource.dynamicFilter = {
            IDATIVO: [{
                relation: 'AND',
                operation: 'IN',
                value: 'S',
            }],
        };
        component.datasource.get();
    }

    public validaVistoria() {
        const vistoria = Metadata.getInstance<Select>('selectCriarVistoria');
        const observacao = Metadata.getInstance<Select>('selectObservacoesObrigatorias');
        const encarregado = Metadata.getInstance<Select>('selectEncarregado1');

        if (vistoria.value === '1') {
            observacao!.datasource!.data!.push({
                value: '3',
                text: 'Somente na vistoria',
            });
            observacao!.datasource!.data!.push({
                value: '4',
                text: 'Na tarefa e na vistoria',
            });
            encarregado.addValidation('required');
            Metadata.getInstance<Select>('selectFuncionario1').value = null;
        } else {
            observacao.datasource?.get();
            encarregado.disabled = true;
            encarregado.removeValidation('required');
        }
        encarregado.value = null;
    }

    /* ================ Funções responsáveis para fazer a inserção das tarefas ===================================== */

    public executorHora(vistoria: string) {
        const dados: any = [];
        const operador = Metadata.getInstance<Select>('selectFuncionario1').value;
        const encarregado = Metadata.getInstance<Select>('selectEncarregado1').value;

        dados.push({
            HORAINICIO: Metadata.getInstance<Time>('selectHoraInicio1').value,
            DURACAO: Metadata.getInstance<TextInput>('selectdDuracao1').value,
            FUNCIONARIO: operador,
            ENCARREGADO: (vistoria === '1' && encarregado === null) ? operador : encarregado,
            NRSEQCONTRATO: Metadata.getInstance<Select>('NRSEQCONTRATO').value,
            NRLOCALCLIENTE: Metadata.getInstance<Select>('NRLOCALCLIENTE').value,
            IDTIPOMARCACAOTAREFA: Metadata.getInstance<Select>('selectOpcoesMarcacao').value,
        });
        return dados;
    }

    public inserirTarefa() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        const checkBoxMultiple = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        const diaSemana = Metadata.getInstance<SelectMultiple>('selectDiaSemana').value;
        const dataRange = Metadata.getInstance<DateRange>('selectData').value;
        const formTarefas = Metadata.getInstance<Form>('formCadastroTarefas');
        const formCriterios = Metadata.getInstance<Form>('formTarefas_dropBoxCriterioAvaliacao');
        const formAgendamento = Metadata.getInstance<Form>('formTarefas_dropBoxDadosAgendamento');
        const vistoria = Metadata.getInstance<Select>('selectCriarVistoria').value;

        if (Utilities.validateForm(formTarefas) && Utilities.validateForm(formCriterios) && Utilities.validateForm(formAgendamento)) {
            LoadingService.show();
            Http.post('/insertTarefasRecorrentes', {
                NOMETAREFA: Metadata.getInstance<TextInput>('nomeTarefa').value,
                CDCLIENTE: Metadata.getInstance<Select>('CDCLIENTE').value,
                NRTIPOLOCALCLI: Metadata.getInstance<SelectMultiple>('NRTIPOLOCALCLI').value,
                NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('NRLOCALCLIENTE').value,
                NRCHECKLIST: Metadata.getInstance<Select>('NRCHECKLIST').value,
                NRATIVIDADECHK: Metadata.getInstance<SelectMultiple>('NRATIVIDADECHK').value,
                NRESCOPOSTAREFA: Metadata.getInstance<SelectMultiple>('NRESCOPOSTAREFA').value,
                NRTAREFACHK: Metadata.getInstance<SelectMultiple>('NRTAREFACHK').value,
                NRCRITERIOTAREFACHK: checkBoxMultiple.datasource?.data.filter((box: any) => checkBoxMultiple.value.indexOf(box.NRCRITERIOTAREFACHK) >= 0),
                DATERANGE: dataRange,
                DIASEMANA: diaSemana,
                VISTORIA: vistoria,
                OBSERVACOES: Metadata.getInstance<Select>('selectObservacoesObrigatorias').value,
                EXECUTOR: this.executorHora(vistoria),
                RECORRENCIA: 'SR',
                CONFLITO: false,
                PARAMETRIZACAO: this.parametrizacao,
                NRPRIORIDADETAREFA: Metadata.getInstance<Select>('select-prioridade').value,
            }).then((response: IDictionary) => {
                if (response.data !== 'OK') {
                    this.tarefaMassaController.abrirModalConflito(response.data);
                    this.fecharModalCadastroTarefas();
                } else {
                    Utilities.showSuccessMessage();
                    this.tarefaMassaController.fecharModalConflito();
                    this.fecharModalCadastroTarefas();
                    grid.datasource.get();
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public excluirTarefas({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00004000TarefasController.aposConfirmarDelete', 'Excluir Registro?');
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<TekGrid>('tek-grid-tarefas');
        LoadingService.show();
        Http.post('/inativarTarefas', {
            NRS: component.selectedRows.map((element: any) => element.NRCHECKLISTAGEND),
        }).then(() => {
            component.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public validaParametrizacao() {
        Http.get('getParametroConflito').then((response) => {
            this.parametrizacao = (response.data.length !== 0) ? response.data[0].IDVALIDACRIATAREFA : null;
        });
    }

    public rowClickFiltroGrid() {
        this.visibilidadeGridItens = true;
        this.visibilidadeForm = !this.visibilidadeGridItens;
    }

    public popularCriteriosModal(nrChecklist: Number, nrAtividadechk: Number, nrTarefachk: Number) {
        const checkbox = Metadata.getInstance<CheckboxMultiple>('NRCRITERIOTAREFACHK');
        checkbox.datasource?.setFilter({
            NRTAREFACHK: nrTarefachk,
            NRATIVIDADECHK: nrAtividadechk,
            NRCHECKLIST: nrChecklist,
            MODAL: true,
        });
        checkbox.datasource?.get();
        checkbox.isVisible = true;
    }

    public formatarDataHora(dtinicio: String, dtfim: String) {
        const [horaini, minutoini] = dtinicio[1].split(':');
        const [horafim, minutofim] = dtfim[1].split(':');
        const hora = (parseInt(horafim, 10) - parseInt(horaini, 10)).toString().padStart(2, '0');
        const minuto = (parseInt(minutofim, 10) - parseInt(minutoini, 10)).toString().padStart(2, '0');
        return `${hora}:${minuto}`;
    }
}
